#define	GPUs                                    \
  TESLA,FERMI,KEPLER,KEPLER2,MAXWELL,MAXWELL2

#define	__func_NAME__( a, b )			\
  API( a##b )

#define	DEF_prototype_by_GPU( func, ... )       \
  __EXPAND_proto__( func, GPUs, GPU_codes )
#define __EXPAND_proto__( func, ... )		\
  __EXPAND_proto( func, __VA_ARGS__)

#define	_prototype( func, GPU )                 \
  extern "C" __host__ void			\
  __func_NAME__( func, GPU ) func##_##args

#define	__EXPAND_proto( func, GPU1, GPU2, GPU3, GPU4, GPU5, GPU6, ... ) \
  _prototype( func, );                                                  \
  _prototype( func, _##GPU1 );                                          \
  _prototype( func, _##GPU2 );                                          \
  _prototype( func, _##GPU3 );                                          \
  _prototype( func, _##GPU4 );                                          \
  _prototype( func, _##GPU5 );                                          \
  _prototype( func, _##GPU6 )



#define	DGEMV_n_args                            \
  ( int blk,					\
    const double alpha, const double beta,	\
    const double * a, const double * x,		\
    double * y,					\
    const int lda,				\
    const int n1, const int n2 )
#define	DGEMV_t_args                            \
  ( int blk,					\
    const double alpha, const double beta,	\
    const double * a, const double * x,		\
    double * y,					\
    const int lda,				\
    const int n1, const int n2 )
#define DSYMVu_LARGE_args                       \
  ( const int blk,				\
    const int n,                                \
    const double * a, const int lda,		\
    const double * x, double * y,               \
    const double alpha, const double beta )
#define DSYMVl_LARGE_args                       \
  ( const int blk,				\
    const int n,                                \
    const double * a, const int lda,		\
    const double * x, double * y,               \
    const double alpha, const double beta )


DEF_prototype_by_GPU( DGEMV_n, GPUs );
DEF_prototype_by_GPU( DGEMV_t, GPUs );
DEF_prototype_by_GPU( DSYMVu_LARGE, GPUs );
DEF_prototype_by_GPU( DSYMVl_LARGE, GPUs );


extern "C"void
API(DSYMVu) ( const int blk,
	      const int n,
	      const double * A, const int lda,
	      const double * x, double * y,
	      const double alpha, const double beta );

extern "C"void
API(DSYMVl) ( const int blk,
	      const int n,
	      const double * A, const int lda,
	      const double * x, double * y,
	      const double alpha, const double beta );

extern "C" __host__ void
API(DSYMVu_lu) ( const int n,
		 const double * a, const int lda,
		 const double * x, double * y,
		 const double alpha, const double beta );

extern "C" __host__ void
API(DSYMVl_lu) ( const int n,
		 const double * a, const int lda,
		 const double * x, double * y,
		 const double alpha, const double beta );

extern "C" __host__ void
API(DTRMV_n_upper_for_DSYMV) (
                              const double alpha, const double beta,
                              const double * a, const double * x,
                              double * y,
                              const int lda,
                              const int n );

extern "C" __host__ void
API(DTRMV_t_upper_for_DSYMV) (
                              const double alpha, const double beta,
                              const double * a, const double * x,
                              double * y,
                              const int lda,
                              const int n );




#define	SGEMV_n_args				\
  ( int blk,                                    \
    const float alpha, const float beta,	\
    const float * a, const float * x,           \
    float * y,                                  \
    const int lda,                              \
    const int n1, const int n2 )
#define	SGEMV_t_args				\
  ( int blk,                                    \
    const float alpha, const float beta,	\
    const float * a, const float * x,           \
    float * y,                                  \
    const int lda,                              \
    const int n1, const int n2 )
#define SSYMVu_LARGE_args			\
  ( const int blk,                              \
    const int n,				\
    const float * a, const int lda,             \
    const float * x, float * y,                 \
    const float alpha, const float beta )
#define SSYMVl_LARGE_args			\
  ( const int blk,                              \
    const int n,				\
    const float * a, const int lda,             \
    const float * x, float * y,                 \
    const float alpha, const float beta )


DEF_prototype_by_GPU( SGEMV_n, GPUs );
DEF_prototype_by_GPU( SGEMV_t, GPUs );
DEF_prototype_by_GPU( SSYMVu_LARGE, GPUs );
DEF_prototype_by_GPU( SSYMVl_LARGE, GPUs );


extern "C" __host__ void
API(SSYMVu) ( const int blk,
	      const int n,
	      const float * A, const int lda,
	      const float * x, float * y,
	      const float alpha, const float beta );

extern "C" __host__ void
API(SSYMVl) ( const int blk,
	      const int n,
	      const float * A, const int lda,
	      const float * x, float * y,
	      const float alpha, const float beta );

extern "C" __host__ void
API(SSYMVu_lu) ( const int n,
		 const float * a, const int lda,
		 const float * x, float * y,
		 const float alpha, const float beta );

extern "C" __host__ void
API(SSYMVl_lu) ( const int n,
		 const float * a, const int lda,
		 const float * x, float * y,
		 const float alpha, const float beta );

extern "C" __host__ void
API(STRMV_n_upper_for_SSYMV) (
                              const float alpha, const float beta,
                              const float * a, const float * x,
                              float * y,
                              const int lda,
                              const int n);

extern "C" __host__ void
API(STRMV_t_upper_for_SSYMV) (
                              const float alpha, const float beta,
                              const float * a, const float * x,
                              float * y,
                              const int lda,
                              const int n);

#undef	__func_NAME__

