#!/bin/bash

IN=$1

Make ()
{
	cd ../../src
	\rm dsymv_upper-*.cu_o
	make -j4
	cd ../bench
	\rm test.o test2.o
	make test-d
	cd ../tuning/dsymvu-current
}

Do ()
{
	# /bin/sh ../d.sh > OK-pat-symv
	echo $IN
	awk -f ../anal_symv.awk $IN | \
	tee anal_symv-result | \
	awk ' \
		/N= '$N'/{ \
			print; \
		} \
	' | \
	sort -r | \
	head -200 | \
	awk ' \
		BEGIN { \
			N=0; \
		} \
		{ \
			if ( N != $2 ) { N=$2; p=1; } \
			print $0" "(int((2000-1)/(p*p))+1); \
			p++; \
		} \
	'
}

Main ()
{
N_PATTERN=`cat IN-exe-a | awk '{ if( i>0 && $1>0 ) printf("%d ",$1); i++; }'`
for N in \
	$N_PATTERN
do
	Do
done
}

DO_MATLAB ()
{
	echo "S=[ 0" > cc$ID.m
	awk ' \
		BEGIN { \
			line = 0; \
		} \
		{ \
			line++; \
			if ( line <= 2 )next; \
			if ( $1 > 0 ) print; \
		} \
	' IN-exe-b >> cc$ID.m
	cat << EOS_M >> cc$ID.m
];

[M,c]=size(S);
N=S(M)-S(1)+1;

EtE = sparse(N,N);
for j=1:M
	k=S(j)-S(1)+1;
	EtE(k,k)=1;
end

D = sparse(N,N);
for j=2:N-1
	D(j,j-1)=1; D(j,j)=-2; D(j,j+1)=1;
end
DtD=N*N*D'*D;

AA=EtE+DtD;

y=[ 0
EOS_M

	awk ' \
		/BLK/{ \
			gsub(/=/," "); \
			flag = ($4=='$ID')?1:0; next; \
		} \
		/N=/{ \
			if(flag>0){ \
				if(flag==1) { \
					flag++; next; \
				} \
				print $2" "$5; flag++; \
			} \
		} \
		' log-dsymv-AU > log-RANK$ID

	awk ' \
		{
			print $2;
		} \
		' log-RANK$ID >> cc$ID.m

	cat << EOS_MM >> cc$ID.m
];

yy=zeros(N,1);
ii=zeros(N,1);
for j=1:M
	s = S(j)-S(1)+1;
	yy(s) = y(j);
end
for j=1:N
	ii(j) = j+S(1)-1;
end

yyN=yy(N);
yy=yy/yyN;
z=(AA\yy)*yyN;

[ii,z]
EOS_MM

	octave cc$ID.m

}

Main | \
	awk '\
		{ \
			s=$4" "$5" "$6" "$7" "$8; \
			score[s] += $9; \
		} \
		END{ \
			for(f in score) \
				printf("%08d %s\n", score[f], f); \
		}' | \
	sort -r | head -20 | tee top20-1step | \
	awk '\
		BEGIN{ \
			id=1; \
		}\
		{ \
			score[id]=$1; \
			BLOCK_SIZE[id]=$2; \
			VX[id]=$3; \
			UX[id]=$4; \
			MULTI[id]=$5; \
			MX[id]=$6; \
			id++; \
		} \
		END { \
			print "{"; \
			print "//=========="; \
			print "if(blk==-1){"; \
			print "\t#include \"dsymv-upper-auto2.h\""; \
			print "}"; \
			print "//=========="; \
			print "#if 0"; \
			print "struct dsymv_auto_tuning_param_t { "; \
			print "\tint\tBLOCK_SIZE, VX, UX, MULTI, MX; "; \
			print "} PARAM["id+1"] = {"; \
				for(i=1;i<id;i++) { \
					print "\t{ "BLOCK_SIZE[i]", "VX[i]", "UX[i]", "MULTI[i]", "MX[i]" },"; \
				} \
			print "\t{ 0, 0, 0, 0, 0 }\n};"; \
			print "#endif"; \
			print "//=========="; \
			print "switch ( BLK ) {"; \
			for(i=1;i<id;i++) { \
			for(m=0;m<10;m++) { \
				print "#if KERNEL_"i*10+m; \
				print "case "i*10+m":"; \
				print "\tAPI(SYMVu_ATOMIC_host) < GPU_ARCH, scalar_t, "BLOCK_SIZE[i]", "VX[i]", "UX[i]", "MULTI[i]", "m" >"; \
				print "\t\t( n, a, lda, x, y, alpha, beta ); break;"; \
				print "#endif"; \
			} \
			} \
			print "default:"; \
			print "\tbreak;"; \
			print "}"; \
			print "//=========="; \
			print "}"; \
		} \
	' > dsymv-upper-auto.h
cp dsymv-upper-auto.h ../


cat top20-1step

touch $IN-X
\rm $IN-X

for ID in \
	1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20
do

\rm dsymv-upper-auto2.h
touch dsymv-upper-auto2.h
awk ' \
	BEGIN{ \
		for(i=0;i<=10;i++) print "#define\tKERNEL_"'$ID'*10+i"\t1"; \
		exit; \
	} \
	' > dsymv-upper-auto2.h
cp dsymv-upper-auto2.h ../

Make >& /dev/null

for M in \
	0 1 2 3 4 5 6 7 8 9
do
	BLOCK_SIZE=`awk '{ i++; if (i=='$ID') { print $2; } }' top20-1step`
	VX=`awk '{ i++; if (i=='$ID') { print $3; } }' top20-1step`
	UX=`awk '{ i++; if (i=='$ID') { print $4; } }' top20-1step`
	MULTIPLICITY=`awk '{ i++; if (i=='$ID') { print $5; } }' top20-1step`

	MM=`awk '{ if ( $1=='$UX'/'$VX' && $2=='$M' ) { print "OK"; } }' OK-pat-symv`

	if [ x$MM != x ]; then

	WW=`cat log-regs-$BLOCK_SIZE-$VX | \
                awk '/^[0-9]/{ if ( $3=='$UX' && $4=='$M') { if ( $NF >= '$MULTIPLICITY' && $8 > 0 ) print "OK"; exit; } }'`

        if [ x$WW != x ]; then

	ulimit -t  720

cat << EOF | tee -a $IN-X
#define	BLOCK_SIZE	$BLOCK_SIZE
#define	VX		$VX
#define	UX		$UX
#define	MULTIPLICITY	$MULTIPLICITY
#define	_M_		$M
EOF

	KERNEL=`expr $ID "*" 10`
	KERNEL=`expr $KERNEL "+" $M`
	../../bench/test2-dsymv-u IN-exe-a $KERNEL | tee -a $IN-X

	fi

	fi
done
done

echo "Complete phase b"
