#!/bin/bash

IN=$1

Make ()
{
	cd ../../src
	\rm ssymv_lower-*.cu_o
	make -j4
	cd ../bench
	\rm test-f.o test2-f.o
	make test-s
	cd ../tuning/ssymvl-current
}

Do ()
{
	# /bin/sh ../d.sh > OK-pat-symv
	echo $IN
	awk -f ../anal_symv.awk $IN | \
	tee anal_symv-result | \
	awk ' \
		/N= '$N'/{ \
			print; \
		} \
	' | \
	sort -r | \
	head -200 | \
	awk ' \
		BEGIN { \
			N=0; \
		} \
		{ \
			if ( N != $2 ) { N=$2; p=1; } \
			print $0" "(int((2000-1)/(p*p))+1); \
			p++; \
		} \
	'
}

Main ()
{
N_PATTERN=`cat IN-exe-a | awk '{ if( i>0 && $1>0 ) printf("%d ",$1); i++; }'`
for N in \
	$N_PATTERN
do
	Do
done
}

DO_MATLAB ()
{
	echo "S=[ 0" > cc$ID.m
	awk ' \
		BEGIN { \
			line = 0; \
		} \
		{ \
			line++; \
			if ( line <= 2 )next; \
			if ( $1 > 0 ) print; \
		} \
	' IN-exe-b >> cc$ID.m
	cat << EOS_M >> cc$ID.m
];

[M,c]=size(S);
N=S(M)-S(1)+1;

EtE = sparse(N,N);
for j=1:M
	k=S(j)-S(1)+1;
	EtE(k,k)=1;
end

D = sparse(N,N);
for j=2:N-1
	D(j,j-1)=1; D(j,j)=-2; D(j,j+1)=1;
end
%DtD=N*N*D'*D;
DtD=N*D'*D;

AA=EtE+DtD;

y=[ 0
EOS_M

	awk ' \
		/BLK/{ \
			gsub(/=/," "); \
			flag = ($4=='$ID')?1:0; next; \
		} \
		/N=/{ \
			if(flag>0){ \
				if(flag==1) { \
					flag++; next; \
				} \
				print $2" "$5; flag++; \
			} \
		} \
		' log-ssymv-AU > log-RANK$ID

	awk ' \
		{
			print $2;
		} \
		' log-RANK$ID >> cc$ID.m

	cat << EOS_MM >> cc$ID.m
];

yy=zeros(N,1);
ii=zeros(N,1);
for j=1:M
	s = S(j)-S(1)+1;
	yy(s) = y(j);
end
for j=1:N
	ii(j) = j+S(1)-1;
end

yyN=yy(N);
yy=yy/yyN;
z=(AA\yy)*yyN;

[ii,z]
EOS_MM

	octave cc$ID.m

}

Main | \
	awk '\
		{ \
			s=$4" "$5" "$6" "$7" "$8; \
			score[s] += $9; \
		} \
		END{ \
			for(f in score) \
				printf("%08d %s\n", score[f], f); \
		}' | \
	sort -r | head -20 | tee top20 | \
	awk '\
		BEGIN{ \
			id=1; \
		}\
		{ \
			score[id]=$1; \
			BLOCK_SIZE[id]=$2; \
			VX[id]=$3; \
			UX[id]=$4; \
			MULTI[id]=$5; \
			MX[id]=$6; \
			id++; \
		} \
		END { \
			print "{"; \
			print "//=========="; \
			print "if(blk==-1){"; \
			print "\t#include \"ssymv-lower-auto2.h\""; \
			print "}"; \
			print "//=========="; \
			print "#if 0"; \
			print "struct ssymv_auto_tuning_param_t { "; \
			print "\tint\tBLOCK_SIZE, VX, UX, MULTI, MX; "; \
			print "} PARAM["id+1"] = {"; \
				for(i=1;i<id;i++) { \
					print "\t{ "BLOCK_SIZE[i]", "VX[i]", "UX[i]", "MULTI[i]", "MX[i]" },"; \
				} \
			print "\t{ 0, 0, 0, 0, 0 }\n};"; \
			print "#endif"; \
			print "//=========="; \
			print "switch ( BLK ) {"; \
			for(i=1;i<id;i++) { \
				print "#if KERNEL_"i; \
				print "case "i":"; \
				print "\tAPI(SYMVl_ATOMIC_host) < GPU_ARCH, scalar_t, "BLOCK_SIZE[i]", "VX[i]", "UX[i]", "MULTI[i]", "MX[i]" >"; \
				print "\t\t( n, a, lda, x, y, alpha, beta ); break;"; \
				print "#endif"; \
			} \
			print "default:"; \
			print "\tbreak;"; \
			print "}"; \
			print "//=========="; \
			print "}"; \
		} \
	' > ssymv-lower-auto.h
\rm ssymv-lower-auto2.h
touch ssymv-lower-auto2.h
awk ' \
	BEGIN{ \
		for(i=0;i<=20;i++) print "#define\tKERNEL_"i"\t1"; \
		exit; \
	} \
	' > ssymv-lower-auto2.h
cp ssymv-lower-auto.h ../
cp ssymv-lower-auto2.h ../

cat top20

Make >& /dev/null

\rm log-ssymv-AU
for ID in \
	1  2  3  4  5  6  7  8  9 10 \
	  11 12 13 14 15 16 17 18 19 20
do
	../../bench/test2-ssymv-l IN-exe-b $ID | tee -a log-ssymv-AU
done

for ID in \
	1  2  3  4  5  6  7  8  9 10 \
	  11 12 13 14 15 16 17 18 19 20
do
	touch mat_mat.$ID
	\rm mat_mat.$ID
	echo 'Fitting the Kernel.'$ID
	DO_MATLAB 1> mat_mat.$ID 2> /dev/null
        awk ' \
		BEGIN { \
			flga=0; \
		} \
		/ans/ { \
			flag=1; next; \
		} \
		/[0-9]/ { \
			if ( flag ) printf("%6d %f\n", $1, $2); \
		} \
	' mat_mat.$ID > mat.$ID
done

echo "Complete phase b2"

